<?php

/**
 * 2007-2019 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2019 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_'))
{

    exit;

}

class Billage extends Module

{

    protected $config_form = false;

    public function __construct()

    {

        $this->name = 'Billage';

        $this->tab = 'checkout';

        $this->version = '1.4.0';

        $this->author = 'Billage';

        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Billage');

        $this->description = $this->l('Sincroniza tus facturas en Billage automáticamente.');

        $this->ps_versions_compliancy = array(
            'min' => '1.6',
            'max' => _PS_VERSION_
        );

    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */

    public function install()

    {

        include (dirname(__FILE__) . '/sql/install.php');

        return parent::install() &&

        $this->registerHook('header') &&

        $this->registerHook('backOfficeHeader') &&

        $this->registerHook('displayOrderConfirmation');

    }

    public function uninstall()

    {

        include (dirname(__FILE__) . '/sql/uninstall.php');

        return parent::uninstall();

    }

    /**
     * Load the configuration form
     */

    public function getContent()

    {

        /**
         * If values have been submitted in the form, process.
         */

        if (((bool)Tools::isSubmit('submitBillageModule')) == true)
        {

            $this->postProcess();

        }

        $this
            ->context
            ->smarty
            ->assign('module_dir', $this->_path);

        $output = $this
            ->context
            ->smarty
            ->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();

    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */

    protected function renderForm()

    {

        $helper = new HelperForm();

        $helper->show_toolbar = false;

        $helper->table = $this->table;

        $helper->module = $this;

        $helper->default_form_language = $this
            ->context
            ->language->id;

        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;

        $helper->submit_action = 'submitBillageModule';

        $helper->currentIndex = $this
            ->context
            ->link
            ->getAdminLink('AdminModules', false)
 . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;

        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(

            'fields_value' => $this->getConfigFormValues() , /* Add values for your inputs */

            'languages' => $this
                ->context
                ->controller
                ->getLanguages() ,

            'id_language' => $this
                ->context
                ->language->id,

        );

        return $helper->generateForm(array(
            $this->getConfigForm()
        ));

    }

    /**
     * Get series
     */

    protected function getBillageSeries($api_key)

    {

        try
        {

            $url = "https://app.getbillage.com/api/v1/invoices/serie?api_key=" . $api_key . "&q=100";

            $ch = curl_init();

            if ($ch === false)
            {

                throw new Exception('failed to initialize');

            }

            curl_setopt($ch, CURLOPT_HEADER, 1);

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

            curl_setopt($ch, CURLINFO_HEADER_OUT, 1);

            curl_setopt($ch, CURLOPT_URL, $url);

            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");

            $result = curl_exec($ch);

            if ($result === false)
            {

                throw new Exception(curl_error($ch) , curl_errno($ch));

            }

            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            curl_close($ch);

            list($header, $body) = explode("\r\n\r\n", $result, 2);

            if (200 == $httpCode)
            {

                return json_decode($body);

            }
            else
            {

                $this->writeWarning('{getBillageSeries Warning #' . $httpCode . '}' . $body . PHP_EOL . '#Fin del Warning#' . PHP_EOL);

            }

        }
        catch(Exception $e)
        {

            trigger_error(sprintf(

            'Curl failed with error #%d: %s',

            $e->getCode() , $e->getMessage()) ,

            E_USER_ERROR);

        }

    }

    protected function getBillageSerie($api_key, $serie_name)

    {

        try
        {

            $url = "https://app.getbillage.com/api/v1/invoices/serie?api_key=" . $api_key . "&name=" . $serie_name;

            $ch = curl_init();

            if ($ch === false)
            {

                throw new Exception('failed to initialize');

            }

            curl_setopt($ch, CURLOPT_HEADER, 1);

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

            curl_setopt($ch, CURLINFO_HEADER_OUT, 1);

            curl_setopt($ch, CURLOPT_URL, $url);

            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");

            $result = curl_exec($ch);

            if ($result === false)
            {

                throw new Exception(curl_error($ch) , curl_errno($ch));

            }

            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            curl_close($ch);

            list($header, $body) = explode("\r\n\r\n", $result, 2);

            if (200 == $httpCode)
            {

                return json_decode($body);

            }
            else
            {

                $this->writeWarning('{getBillageSerie Warning #' . $httpCode . '}' . $body . PHP_EOL . '#Fin del Warning#' . PHP_EOL);

            }

        }
        catch(Exception $e)
        {

            trigger_error(sprintf(

            'Curl failed with error #%d: %s',

            $e->getCode() , $e->getMessage()) ,

            E_USER_ERROR);

        }

    }

    protected function postBillage($api_key, $url, $data, $code)

    {

        try
        {

            $ch = curl_init();

            if ($ch === false)
            {

                throw new Exception('failed to initialize');

            }

            $encoded_data = json_encode($data);

            curl_setopt($ch, CURLOPT_HEADER, 1);

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

            curl_setopt($ch, CURLINFO_HEADER_OUT, 1);

            curl_setopt($ch, CURLOPT_URL, $url);

            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");

            curl_setopt($ch, CURLOPT_POSTFIELDS, $encoded_data);

            curl_setopt($ch, CURLOPT_HTTPHEADER, array(

                'Content-Type: application/json;charset=UTF-8',

                'Content-Length: ' . strlen($encoded_data) ,

                'x-api-key: ' . $api_key,

            )
);

            $result = curl_exec($ch);

            if ($result === false)
            {

                throw new Exception(curl_error($ch) , curl_errno($ch));

            }

            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            curl_close($ch);

            list($header, $body) = explode("\r\n\r\n", $result, 2);

            if (200 == $httpCode)
            {

                return json_decode($body);

            }
            else
            {

                $this->writeWarning('Factura número #' . $code . PHP_EOL . $body . PHP_EOL . '#Fin del Warning#' . PHP_EOL);

            }

        }
        catch(Exception $e)
        {

            trigger_error(sprintf(

            'Curl failed with error #%d: %s',

            $e->getCode() , $e->getMessage()) ,

            E_USER_ERROR);

        }

    }

    protected function getBillageCustomer($api_key, $customer)

    {

        try
        {

            $spaceless_customer = str_replace(' ', '%20', $customer);

            $url = "https://app.getbillage.com/api/v2/accounts?api_key=" . $api_key . "&q=" . $spaceless_customer . "&start=1&elements=20";

            $ch = curl_init();

            if ($ch === false)
            {

                throw new Exception('failed to initialize');

            }

            curl_setopt($ch, CURLOPT_HEADER, 1);

            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

            curl_setopt($ch, CURLINFO_HEADER_OUT, 1);

            curl_setopt($ch, CURLOPT_URL, $url);

            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");

            $result = curl_exec($ch);

            if ($result === false)
            {

                throw new Exception(curl_error($ch) , curl_errno($ch));

            }

            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            curl_close($ch);

            list($header, $body) = explode("\r\n\r\n", $result, 2);

            if (200 == $httpCode)
            {

                return json_decode($body);

            }
            else
            {

                $this->writeWarning('{getBillageCustomer Warning #' . $httpCode . '}' . $body);

            }

        }
        catch(Exception $e)
        {

            trigger_error(sprintf(

            'Curl failed with error #%d: %s',

            $e->getCode() , $e->getMessage()) ,

            E_USER_ERROR);

        }

    }

    protected function getBillageTaxName($tax_rate)

    {

        switch ($tax_rate)
        {

            case 0:

                $result = 'ES_IVA_EXENTO_0.00_0.00';

            break;

            case 3:

                $result = 'ES_IGIC_3.00_0.00';

            break;

            case 4:

                $result = 'ES_IVA_4.00_0.50';

            break;

            case 6.5:

                $result = 'ES_IGIC_6.50_0.00';

            break;

            case 7:

                $result = 'ES_IVA_7.00_0.00';

            break;

            case 10:

                $result = 'ES_IVA_10.00_1.40';

            break;

            case 18:

                $result = 'ES_IVA_18.00_4.00';

            break;

            case 21:

                $result = 'ES_IVA_21.00_0.00';

            break;

            default:

                $result = 'ES_IVA_21.00_0.00';

            break;

        }

        return $result;

    }

    protected function writeError($code, $postStatus, $postErrorMessage)

    {

        $module_path = $_SERVER['DOCUMENT_ROOT'] . '/modules/Billage/log/error_log.txt';

        $write_error = fopen($module_path, 'a');

        $error_message = date("Y-m-d H:i:s") . " - Factura número " . $code . ". Error " . $postStatus . ": " . $postErrorMessage . "}" . PHP_EOL;

        fwrite($write_error, $error_message);

        fclose($write_error);

    }

    protected function writeWarning($message)

    {

        $module_path = $_SERVER['DOCUMENT_ROOT'] . '/modules/Billage/log/error_log.txt';

        $write_error = fopen($module_path, 'a');

        $error_message = date("Y-m-d H:i:s") . " - Warning: " . $message . PHP_EOL;

        fwrite($write_error, $error_message);

        fclose($write_error);

    }

    /**
     * Create the structure of your form.
     */

    protected function getConfigForm()

    {

        $api_key = Configuration::get('BILLAGE_API_KEY');

        if ($api_key)
        {

            $allSeries = $this->getBillageSeries($api_key);

            foreach ($allSeries->page as $loopSerie => $aliasSerie)
            {

                $options[] = array(

                    'id_option' => $aliasSerie->alias,

                    'name' => $aliasSerie->alias,

                );

            }

        }

        $configForm = array(

            'form' => array(

                'legend' => array(

                    'title' => $this->l('Configuración') ,

                    'icon' => 'icon-cogs',

                ) ,

                'input' => array(

                    array(

                        'col' => 3,

                        'type' => 'text',

                        'name' => 'BILLAGE_API_KEY',

                        'desc' => $this->l('Rellena la API Key para poder seleccionar una serie.') ,

                        'label' => $this->l('API Key') ,

                    ) ,

                ) ,

                'submit' => array(

                    'title' => $this->l('Save') ,

                ) ,

            ) ,

        );

        if ($api_key)
        {

            $configForm['form']['input'][] = array(

                'type' => 'select',

                'lang' => true,

                'label' => $this->l('Series') ,

                'name' => 'BILLAGE_SERIE',

                'desc' => $this->l('Por favor selecciona una serie. Recuerda volver a guardar la configuración.') ,

                'options' => array(

                    'query' => $options,

                    'id' => 'id_option',

                    'name' => 'name'

                ) ,

            );

        }

        return $configForm;

    }

    /**
     * Set values for the inputs.
     */

    protected function getConfigFormValues()

    {

        return array(

            'BILLAGE_API_KEY' => Configuration::get('BILLAGE_API_KEY', null) ,

            'BILLAGE_SERIE' => Configuration::get('BILLAGE_SERIE', null) ,

        );

    }

    /**
     * Save form data.
     */

    protected function postProcess()

    {

        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key)
        {

            Configuration::updateValue($key, Tools::getValue($key));

        }

    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */

    public function hookBackOfficeHeader()

    {

        if (Tools::getValue('module_name') == $this->name)
        {

            $this
                ->context
                ->controller
                ->addJS($this->_path . 'views/js/back.js');

            $this
                ->context
                ->controller
                ->addCSS($this->_path . 'views/css/back.css');

        }

    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */

    public function hookHeader()

    {

        $this
            ->context
            ->controller
            ->addJS($this->_path . '/views/js/front.js');

        $this
            ->context
            ->controller
            ->addCSS($this->_path . '/views/css/front.css');

    }

    public function hookDisplayOrderConfirmation()

    {

        $api_key = Configuration::get('BILLAGE_API_KEY');

        if ($api_key != null)
        {

            $currentSerie = Configuration::get('BILLAGE_SERIE');

            if ($currentSerie == null) $currentSerie = "PrestaShop";

            $id_order = Tools::getValue('id_order');

            $order = new Order($id_order);

            $fields = $order->getFields();

            $products = $order->getProducts();

            $address = new Address($this
                ->context
                ->cart
                ->id_address_delivery);

            $raw_date = $fields['date_add'];

            $invoiceDate = date("Y-m-d", strtotime($raw_date));

            $code = $fields['id_order'];

            $internal_notes = 'Pedido generado a través de Prestashop. Número de pedido #' . $code;

            switch ($fields['current_state'])
            {

                case 2:

                    $state = 'charged';

                break;

                case 9:

                    $state = 'charged';

                break;

                case 11:

                    $state = 'pending';

                break;

                default:

                    $state = 'pending';

                break;

            }

            $currency = 'EUR';

            $email = $this
                ->context
                ->customer->email;

            $phone = $address->phone;

            $phone_mobile = $address->phone_mobile;

            $dni = $address->dni;

            $billing_address = $address->address1 . ' ' . $address->address2;

            $zip_code = $address->postcode;

            $city = $address->city;

            $country = $address->country;

            if (empty($address->lastname))
            {

                $customer = $address->firstname;

            }
            else
            {

                $customer = $address->firstname . " " . $address->lastname;

            }

            if ($address->company == '')
            {

                $company = $customer;

            }
            else
            {

                $company = $address->company;

            }

            $payment_method = $fields['module'];

            $payment_document = $fields['payment'];

            $position = 1;

            foreach ($products as $product_item => $product_value)
            {

                $product_iva = $this->getBillageTaxName($product_value['tax_rate']);

                $concepts[] = array(

                    'id' => null,

                    'type' => 'product',

                    'description' => $product_value['product_name'],

                    'discount' => 0.0,

                    'notes' => '',

                    'position' => $position++,

                    'price' => $product_value['product_price'],

                    'quantity' => $product_value['product_quantity'],

                    'tax' => $product_iva,

                    'total' => $product_value['product_price'],

                    'retention' => null,

                    'unit' => null,

                    'rounding' => false,

                );

            }

            $shipping_iva = $this->getBillageTaxName($fields['carrier_tax_rate']);

            $concepts[] = array(

                'id' => null,

                'type' => 'product',

                'description' => 'Tarifa de envio',

                'discount' => 0.0,

                'notes' => '',

                'position' => $position,

                'price' => $fields['total_shipping_tax_excl'],

                'quantity' => 1,

                'tax' => $shipping_iva,

                'total' => $fields['total_shipping_tax_excl'],

                'retention' => null,

                'unit' => null,

                'rounding' => false,

            );

            $getSerieStatus = $this->getBillageSerie($api_key, 'PrestaShop');

            if (!$getSerieStatus)
            {

                $postSerie = array(

                    'alias' => "PrestaShop",

                    'by_default' => false,

                    'prefix' => 'PS',

                    'sales_document_type' => array(

                        'id' => 2,

                    ) ,

                );

                $this->postBillage($api_key, 'https://app.getbillage.com/api/v1/invoices/serie', $postSerie, $code);

            }

            // Data de la factura
            

            $invoice_data = array(

                'api_key' => $api_key,

                'id' => null,

                'date' => $invoiceDate,

                'expirationDate' => $invoiceDate,

                'type' => 'normal',

                'state' => $state,

                'internal_notes' => $internal_notes,

                'currency' => $currency,

                'serie' => $currentSerie,

                'category' => null,

                'customer' => $customer,

                'language' => 'Castellano',

                'surcharge_equivalence' => false,

                'concepts' => $concepts,

                'payment_method' => 'CASH',

                'payment_document' => 'CASH',

                'delivery_address' => null,

                'company_direction' => null,

            );

            $billageCustomer = $this->getBillageCustomer($api_key, $customer);

            if ($billageCustomer->page_elements == 1)
            {

                if ($billageCustomer->page[0]->email == $email)
                {

                    $this->postBillage($api_key, 'https://app.getbillage.com/api/v1/invoices', $invoice_data, $code);

                }
                else
                {

                    $this->writeWarning('Factura número #' . $code . 'Ya existe un usuario con este Alias en Billage por lo que no se ha podido crear ni añadir su factura.');

                }

            }
            elseif ($billageCustomer->page_elements == 0)
            {

                // Data del nuevo cliente
                

                $customer_data = array(

                    'api_key' => $api_key,

                    'addresses' => array(
                        [

                        'account' => array(

                            'alias' => $customer,

                        ) ,

                        'address' => $billing_address,

                        'city' => $city,

                        'country' => $country,

                        'email' => $email,

                        'zip_code' => $zip_code,

                        ]
                    ) ,

                    'alias' => $customer,

                    'business_name' => $company,

                    'email' => $email,

                    'phones' => array(

                        $phone,

                    ) ,

                    'vat' => $dni,

                );

                // Post del cliente
                

                $this->postBillage($api_key, 'https://app.getbillage.com/api/v1/accounts', $customer_data, $code);

                // Post de la factura
                

                $this->postBillage($api_key, 'https://app.getbillage.com/api/v1/invoices', $invoice_data, $code);

            }

        }

    }

}